/*
    Copyright 2010 Computational Electromagnetics Research Laboratory (CERL)
    University of Victoria, Victoria, B.C. Canada
    http://www.cerl.ece.uvic.ca
    http://www.cerl.ece.uvic.ca/Open_Source_CEM.htm
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//////////////////////////////////////////////////////////////////////////////
//
// CLASS TEMPLATE: Array3D
//
// This is a generic three-dimensional array class.  Objects of this class
// can be treated as two- and one-dimensional arrays.  Array elements can be
// accessed using the () operator.  It is not necessary to use this class
// for one-dimensional array if the bulit-in C/C++ array meet your need.  
//

#ifdef _Array3D_Examples_
// Examples
// --------
Array3D<int> a(10,10,10), b(a); // Create two 3D arrays of int, "a" and "b"
                                // have the same size (10x10x10) and
                                // dimension 3D
a = 0;                          // Assign "0" to all elements in "a"
b = a;                          // Copy values of "a" to "b"
b(3,4,5) = 7;                   // Assign a value to a specific entry in "b"

Array3D<double> c(10,10,1);     // Create a 2D array of double
c(2,1) = 4.0;                   // An example ssignment statement

Array3D<char> d(10,1,1);        // Create a 1D array of char
d = 'e';                        // Assign 'e' to all elments in "d"; note
                                // that this is not possible if "d" is 
                                // created using the built-in C/C++ syntax
                                // char d[10]
d(7) = 'k';                     // Change the 8th element to 'k'
#endif
//
//////////////////////////////////////////////////////////////////////////////
#pragma once

#include <assert.h>

template<class T>
class Array3D
{
public:
	Array3D(const Array3D<T> &a)
	: i_size(a.i_size),j_size(a.j_size),k_size(a.k_size),b_size(a.b_size)
	{
		buffer = new T[b_size];
		*this = a;
	}
	Array3D(int I, int J, int K)
	: i_size(I),j_size(J),k_size(K),b_size(I*J*K)
	{
		buffer = new T[b_size];
	}
	~Array3D()
	{
		delete[] buffer;
	}
	T& operator()(int i, int j, int k)const
	{
		assert(i>=0 && i<i_size && j>=0 && j<j_size && k>=0 && k<k_size);
		return buffer[(i*j_size+j)*k_size+k];
	}
	T& operator()(int i, int j)const
	{
		assert(i>=0 && i<i_size && j>=0 && j<j_size && k_size==1);
		return buffer[i*j_size+j];
	}
	T& operator()(int i)const
	{
		assert(i>=0 && i<i_size && j_size==1 && k_size==1);
		return buffer[i];
	}
	T& operator[](int i)const
	{
		assert(i>=0 && i<i_size && j_size==1 && k_size==1);
		return buffer[i];
	}

	Array3D<T>& operator=(const Array3D<T>& a)
	{
		assert (i_size==a.i_size && j_size==a.j_size &&
				k_size==a.k_size &&	b_size==a.b_size);
		for (int n=0; n<b_size; n++) buffer[n]=a.buffer[n];
		return *this;
	}
	Array3D<T>& operator=(const T &v)
	{
		for (int n=0; n<b_size; n++) buffer[n]=v;
		return *this;
	}
	operator T*(){ return buffer; }
	operator const T*() const { return buffer; }

	const int i_size,j_size,k_size, b_size;

protected:
	T *buffer;
};
typedef Array3D<double> Double3D;
